import agilicus
from . import context
from .input_helpers import build_updated_model
from .input_helpers import pop_item_if_none


def query(ctx, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    result = apiclient.catalogues_api.list_catalogues(**kwargs)
    if result:
        return result.catalogues


def add(ctx, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    model = agilicus.Catalogue(**kwargs)
    return apiclient.catalogues_api.create_catalogue(model).to_dict()


def show(ctx, catalogue_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.catalogues_api.get_catalogue(catalogue_id).to_dict()


def _build_updated_catalogue(catalogue):
    new_catalogue_dict = catalogue.to_dict()
    new_catalogue_dict.pop("catalogue_entries", [])
    return agilicus.Catalogue(**new_catalogue_dict)


def update(ctx, catalogue_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    catalogue = apiclient.catalogues_api.get_catalogue(catalogue_id)
    catalogue = _build_updated_catalogue(catalogue)
    catalogue = build_updated_model(agilicus.Catalogue, catalogue, kwargs)
    return apiclient.catalogues_api.replace_catalogue(catalogue_id, catalogue).to_dict()


def delete(ctx, catalogue_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.catalogues_api.delete_catalogue(catalogue_id)


def query_entries(ctx, catalogue_id=None, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    if catalogue_id:
        # The id and category are tied as a result this method doesn't take a category
        kwargs.pop("catalogue_category", None)
        result = apiclient.catalogues_api.list_catalogue_entries(catalogue_id, **kwargs)
    else:
        result = apiclient.catalogues_api.list_all_catalogue_entries(**kwargs)

    if result:
        return result.catalogue_entries


def add_entry(ctx, catalogue_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    pop_item_if_none(kwargs, "tag")
    model = agilicus.CatalogueEntry(**kwargs)
    return apiclient.catalogues_api.create_catalogue_entry(catalogue_id, model).to_dict()


def show_entry(ctx, catalogue_id, entry_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.catalogues_api.get_catalogue_entry(catalogue_id, entry_id).to_dict()


def update_entry(ctx, catalogue_id, entry_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    catalogue_entry = apiclient.catalogues_api.get_catalogue_entry(
        catalogue_id, entry_id
    )
    catalogue_entry = build_updated_model(
        agilicus.CatalogueEntry, catalogue_entry, kwargs
    )
    return apiclient.catalogues_api.replace_catalogue_entry(
        catalogue_id, entry_id, catalogue_entry
    ).to_dict()


def delete_entry(ctx, catalogue_id, entry_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.catalogues_api.delete_catalogue_entry(catalogue_id, entry_id)



# flake8: noqa

"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2021.06.02
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""



# import ApiClient
from agilicus_api.api_client import ApiClient

# import Configuration
from agilicus_api.configuration import Configuration

from agilicus_api.api.application_services_api import ApplicationServicesApi
from agilicus_api.api.applications_api import ApplicationsApi
from agilicus_api.api.audits_api import AuditsApi
from agilicus_api.api.catalogues_api import CataloguesApi
from agilicus_api.api.certificates_api import CertificatesApi
from agilicus_api.api.challenges_api import ChallengesApi
from agilicus_api.api.connectors_api import ConnectorsApi
from agilicus_api.api.diagnostics_api import DiagnosticsApi
from agilicus_api.api.files_api import FilesApi
from agilicus_api.api.groups_api import GroupsApi
from agilicus_api.api.issuers_api import IssuersApi
from agilicus_api.api.messages_api import MessagesApi
from agilicus_api.api.metrics_api import MetricsApi
from agilicus_api.api.organisations_api import OrganisationsApi
from agilicus_api.api.permissions_api import PermissionsApi
from agilicus_api.api.policy_api import PolicyApi
from agilicus_api.api.resources_api import ResourcesApi
from agilicus_api.api.services_api import ServicesApi
from agilicus_api.api.tokens_api import TokensApi
from agilicus_api.api.users_api import UsersApi
from agilicus_api.api.whoami_api import WhoamiApi


# import exceptions
from agilicus_api.exceptions import OpenApiException
#from agilicus_api.exceptions import ApiAttributeError
from agilicus_api.exceptions import ApiTypeError
from agilicus_api.exceptions import ApiValueError
from agilicus_api.exceptions import ApiKeyError
from agilicus_api.exceptions import ApiException

from agilicus_api.model.api_key import APIKey
from agilicus_api.model.api_key_introspect import APIKeyIntrospect
from agilicus_api.model.api_key_introspect_authorization_info import APIKeyIntrospectAuthorizationInfo
from agilicus_api.model.api_key_introspect_response import APIKeyIntrospectResponse
from agilicus_api.model.api_key_spec import APIKeySpec
from agilicus_api.model.api_key_status import APIKeyStatus
from agilicus_api.model.access_requests import AccessRequests
from agilicus_api.model.access_requests_status import AccessRequestsStatus
from agilicus_api.model.add_group_member_request import AddGroupMemberRequest
from agilicus_api.model.agent_connector import AgentConnector
from agilicus_api.model.agent_connector_authz_stats import AgentConnectorAuthzStats
from agilicus_api.model.agent_connector_connection_info import AgentConnectorConnectionInfo
from agilicus_api.model.agent_connector_info import AgentConnectorInfo
from agilicus_api.model.agent_connector_per_share_stats import AgentConnectorPerShareStats
from agilicus_api.model.agent_connector_proxy_request_stats import AgentConnectorProxyRequestStats
from agilicus_api.model.agent_connector_proxy_request_stats_details import AgentConnectorProxyRequestStatsDetails
from agilicus_api.model.agent_connector_proxy_stats import AgentConnectorProxyStats
from agilicus_api.model.agent_connector_share_stats import AgentConnectorShareStats
from agilicus_api.model.agent_connector_spec import AgentConnectorSpec
from agilicus_api.model.agent_connector_specific_stats import AgentConnectorSpecificStats
from agilicus_api.model.agent_connector_stats import AgentConnectorStats
from agilicus_api.model.agent_connector_status import AgentConnectorStatus
from agilicus_api.model.agent_connector_system_stats import AgentConnectorSystemStats
from agilicus_api.model.agent_connector_transport_stats import AgentConnectorTransportStats
from agilicus_api.model.agent_local_auth_info import AgentLocalAuthInfo
from agilicus_api.model.allow_map_compiled import AllowMapCompiled
from agilicus_api.model.allow_rule_compiled import AllowRuleCompiled
from agilicus_api.model.application import Application
from agilicus_api.model.application_assignment import ApplicationAssignment
from agilicus_api.model.application_authentication_config import ApplicationAuthenticationConfig
from agilicus_api.model.application_config import ApplicationConfig
from agilicus_api.model.application_monitoring_config import ApplicationMonitoringConfig
from agilicus_api.model.application_security import ApplicationSecurity
from agilicus_api.model.application_service import ApplicationService
from agilicus_api.model.application_service_assignment import ApplicationServiceAssignment
from agilicus_api.model.application_service_stats import ApplicationServiceStats
from agilicus_api.model.application_service_stats_group import ApplicationServiceStatsGroup
from agilicus_api.model.application_summary import ApplicationSummary
from agilicus_api.model.application_summary_status import ApplicationSummaryStatus
from agilicus_api.model.audit import Audit
from agilicus_api.model.auth_audits import AuthAudits
from agilicus_api.model.authentication_attribute import AuthenticationAttribute
from agilicus_api.model.authentication_document import AuthenticationDocument
from agilicus_api.model.authentication_document_spec import AuthenticationDocumentSpec
from agilicus_api.model.authentication_document_status import AuthenticationDocumentStatus
from agilicus_api.model.auto_create_status import AutoCreateStatus
from agilicus_api.model.base_upstream import BaseUpstream
from agilicus_api.model.base_upstreams import BaseUpstreams
from agilicus_api.model.bulk_session_operation_response import BulkSessionOperationResponse
from agilicus_api.model.bulk_token_revoke import BulkTokenRevoke
from agilicus_api.model.bulk_token_revoke_response import BulkTokenRevokeResponse
from agilicus_api.model.bulk_user_metadata import BulkUserMetadata
from agilicus_api.model.cors_origin import CORSOrigin
from agilicus_api.model.cors_settings import CORSSettings
from agilicus_api.model.csp_directive import CSPDirective
from agilicus_api.model.csp_settings import CSPSettings
from agilicus_api.model.csr_reason_enum import CSRReasonEnum
from agilicus_api.model.catalogue import Catalogue
from agilicus_api.model.catalogue_entry import CatalogueEntry
from agilicus_api.model.cert_signing_req import CertSigningReq
from agilicus_api.model.cert_signing_req_spec import CertSigningReqSpec
from agilicus_api.model.cert_signing_req_status import CertSigningReqStatus
from agilicus_api.model.certificate_transparency_settings import CertificateTransparencySettings
from agilicus_api.model.challenge import Challenge
from agilicus_api.model.challenge_answer import ChallengeAnswer
from agilicus_api.model.challenge_answer_spec import ChallengeAnswerSpec
from agilicus_api.model.challenge_endpoint import ChallengeEndpoint
from agilicus_api.model.challenge_spec import ChallengeSpec
from agilicus_api.model.challenge_status import ChallengeStatus
from agilicus_api.model.cipher_diffie_hellman_group import CipherDiffieHellmanGroup
from agilicus_api.model.cipher_encryption_algorithm import CipherEncryptionAlgorithm
from agilicus_api.model.cipher_integrity_algorithm import CipherIntegrityAlgorithm
from agilicus_api.model.combined_rules import CombinedRules
from agilicus_api.model.combined_rules_status import CombinedRulesStatus
from agilicus_api.model.combined_user_detail import CombinedUserDetail
from agilicus_api.model.combined_user_detail_status import CombinedUserDetailStatus
from agilicus_api.model.common_metadata import CommonMetadata
from agilicus_api.model.connector import Connector
from agilicus_api.model.connector_diagnostic_stats import ConnectorDiagnosticStats
from agilicus_api.model.connector_spec import ConnectorSpec
from agilicus_api.model.connector_stats import ConnectorStats
from agilicus_api.model.connector_stats_metadata import ConnectorStatsMetadata
from agilicus_api.model.connector_status import ConnectorStatus
from agilicus_api.model.content_type_options_settings import ContentTypeOptionsSettings
from agilicus_api.model.create_session_and_token_request import CreateSessionAndTokenRequest
from agilicus_api.model.create_session_and_token_response import CreateSessionAndTokenResponse
from agilicus_api.model.create_token_request import CreateTokenRequest
from agilicus_api.model.cross_origin_embedder_policy_settings import CrossOriginEmbedderPolicySettings
from agilicus_api.model.cross_origin_opener_policy_settings import CrossOriginOpenerPolicySettings
from agilicus_api.model.cross_origin_resource_policy_settings import CrossOriginResourcePolicySettings
from agilicus_api.model.definition import Definition
from agilicus_api.model.desktop_client_configuration import DesktopClientConfiguration
from agilicus_api.model.desktop_client_generated_configuration import DesktopClientGeneratedConfiguration
from agilicus_api.model.desktop_resource import DesktopResource
from agilicus_api.model.desktop_resource_spec import DesktopResourceSpec
from agilicus_api.model.desktop_resource_status import DesktopResourceStatus
from agilicus_api.model.domain import Domain
from agilicus_api.model.email import Email
from agilicus_api.model.environment import Environment
from agilicus_api.model.environment_config import EnvironmentConfig
from agilicus_api.model.environment_config_var import EnvironmentConfigVar
from agilicus_api.model.environment_status import EnvironmentStatus
from agilicus_api.model.error_message import ErrorMessage
from agilicus_api.model.feature_flag import FeatureFlag
from agilicus_api.model.file import File
from agilicus_api.model.file_name import FileName
from agilicus_api.model.file_share_service import FileShareService
from agilicus_api.model.file_share_service_spec import FileShareServiceSpec
from agilicus_api.model.file_share_service_stats import FileShareServiceStats
from agilicus_api.model.file_share_service_stats_group import FileShareServiceStatsGroup
from agilicus_api.model.file_share_service_status import FileShareServiceStatus
from agilicus_api.model.file_summary import FileSummary
from agilicus_api.model.file_visibility import FileVisibility
from agilicus_api.model.frame_options_settings import FrameOptionsSettings
from agilicus_api.model.generic_float_metric import GenericFloatMetric
from agilicus_api.model.generic_int_metric import GenericIntMetric
from agilicus_api.model.group import Group
from agilicus_api.model.group_data import GroupData
from agilicus_api.model.group_member import GroupMember
from agilicus_api.model.group_reconcile_record import GroupReconcileRecord
from agilicus_api.model.hsts_settings import HSTSSettings
from agilicus_api.model.http_security_settings import HTTPSecuritySettings
from agilicus_api.model.host_permissions import HostPermissions
from agilicus_api.model.http_rule import HttpRule
from agilicus_api.model.identity_assertion import IdentityAssertion
from agilicus_api.model.identity_assertion_response import IdentityAssertionResponse
from agilicus_api.model.included_role import IncludedRole
from agilicus_api.model.ipsec_connection import IpsecConnection
from agilicus_api.model.ipsec_connection_ipv4_block import IpsecConnectionIpv4Block
from agilicus_api.model.ipsec_connection_spec import IpsecConnectionSpec
from agilicus_api.model.ipsec_connector import IpsecConnector
from agilicus_api.model.ipsec_connector_spec import IpsecConnectorSpec
from agilicus_api.model.ipsec_connector_status import IpsecConnectorStatus
from agilicus_api.model.ipsec_gateway_interface import IpsecGatewayInterface
from agilicus_api.model.issuer import Issuer
from agilicus_api.model.issuer_client import IssuerClient
from agilicus_api.model.json_body_constraint import JSONBodyConstraint
from agilicus_api.model.k8s_slug import K8sSlug
from agilicus_api.model.list_api_keys_response import ListAPIKeysResponse
from agilicus_api.model.list_access_requests_response import ListAccessRequestsResponse
from agilicus_api.model.list_active_users_response import ListActiveUsersResponse
from agilicus_api.model.list_agent_connector_response import ListAgentConnectorResponse
from agilicus_api.model.list_application_services_response import ListApplicationServicesResponse
from agilicus_api.model.list_application_summary_response import ListApplicationSummaryResponse
from agilicus_api.model.list_applications_response import ListApplicationsResponse
from agilicus_api.model.list_audits_response import ListAuditsResponse
from agilicus_api.model.list_auth_audits_response import ListAuthAuditsResponse
from agilicus_api.model.list_authentication_document_response import ListAuthenticationDocumentResponse
from agilicus_api.model.list_catalogue_entries_response import ListCatalogueEntriesResponse
from agilicus_api.model.list_catalogues_response import ListCataloguesResponse
from agilicus_api.model.list_cert_signing_req_response import ListCertSigningReqResponse
from agilicus_api.model.list_combined_rules_response import ListCombinedRulesResponse
from agilicus_api.model.list_combined_user_details_response import ListCombinedUserDetailsResponse
from agilicus_api.model.list_configs_response import ListConfigsResponse
from agilicus_api.model.list_connector_response import ListConnectorResponse
from agilicus_api.model.list_desktop_resources_response import ListDesktopResourcesResponse
from agilicus_api.model.list_elevated_user_roles import ListElevatedUserRoles
from agilicus_api.model.list_environment_configs_response import ListEnvironmentConfigsResponse
from agilicus_api.model.list_file_share_services_response import ListFileShareServicesResponse
from agilicus_api.model.list_files_response import ListFilesResponse
from agilicus_api.model.list_groups_response import ListGroupsResponse
from agilicus_api.model.list_ipsec_connector_response import ListIpsecConnectorResponse
from agilicus_api.model.list_issuer_clients_response import ListIssuerClientsResponse
from agilicus_api.model.list_issuer_extensions_response import ListIssuerExtensionsResponse
from agilicus_api.model.list_issuer_roots_response import ListIssuerRootsResponse
from agilicus_api.model.list_logs_response import ListLogsResponse
from agilicus_api.model.list_mfa_challenge_methods import ListMFAChallengeMethods
from agilicus_api.model.list_message_endpoints_response import ListMessageEndpointsResponse
from agilicus_api.model.list_orgs_response import ListOrgsResponse
from agilicus_api.model.list_policies_response import ListPoliciesResponse
from agilicus_api.model.list_policy_rules_response import ListPolicyRulesResponse
from agilicus_api.model.list_resource_permissions_response import ListResourcePermissionsResponse
from agilicus_api.model.list_resource_roles_response import ListResourceRolesResponse
from agilicus_api.model.list_resources_response import ListResourcesResponse
from agilicus_api.model.list_role_to_rule_entries import ListRoleToRuleEntries
from agilicus_api.model.list_roles import ListRoles
from agilicus_api.model.list_rules import ListRules
from agilicus_api.model.list_secure_agent_response import ListSecureAgentResponse
from agilicus_api.model.list_service_account_response import ListServiceAccountResponse
from agilicus_api.model.list_service_forwarders_response import ListServiceForwardersResponse
from agilicus_api.model.list_services_response import ListServicesResponse
from agilicus_api.model.list_sessions_response import ListSessionsResponse
from agilicus_api.model.list_totp_enrollment_response import ListTOTPEnrollmentResponse
from agilicus_api.model.list_tokens_response import ListTokensResponse
from agilicus_api.model.list_top_users_response import ListTopUsersResponse
from agilicus_api.model.list_upstream_group_mapping import ListUpstreamGroupMapping
from agilicus_api.model.list_upstream_user_identities_response import ListUpstreamUserIdentitiesResponse
from agilicus_api.model.list_user_application_access_info_response import ListUserApplicationAccessInfoResponse
from agilicus_api.model.list_user_file_share_access_info_response import ListUserFileShareAccessInfoResponse
from agilicus_api.model.list_user_guids_response import ListUserGuidsResponse
from agilicus_api.model.list_user_metadata_response import ListUserMetadataResponse
from agilicus_api.model.list_user_request_info_response import ListUserRequestInfoResponse
from agilicus_api.model.list_user_resource_access_info_response import ListUserResourceAccessInfoResponse
from agilicus_api.model.list_user_roles_for_an_org import ListUserRolesForAnOrg
from agilicus_api.model.list_users_response import ListUsersResponse
from agilicus_api.model.list_web_auth_n_enrollment_response import ListWebAuthNEnrollmentResponse
from agilicus_api.model.list_well_known_issuer_info import ListWellKnownIssuerInfo
from agilicus_api.model.list_x509_certificate_response import ListX509CertificateResponse
from agilicus_api.model.local_auth_upstream_config import LocalAuthUpstreamConfig
from agilicus_api.model.local_auth_upstream_identity_provider import LocalAuthUpstreamIdentityProvider
from agilicus_api.model.log import Log
from agilicus_api.model.login_session import LoginSession
from agilicus_api.model.mfa_challenge_answer import MFAChallengeAnswer
from agilicus_api.model.mfa_challenge_answer_result import MFAChallengeAnswerResult
from agilicus_api.model.mfa_challenge_method import MFAChallengeMethod
from agilicus_api.model.mfa_challenge_method_spec import MFAChallengeMethodSpec
from agilicus_api.model.mfa_challenge_question import MFAChallengeQuestion
from agilicus_api.model.mfa_challenge_question_input import MFAChallengeQuestionInput
from agilicus_api.model.mfa_challenge_question_login_info import MFAChallengeQuestionLoginInfo
from agilicus_api.model.mfa_enrollment_answer import MFAEnrollmentAnswer
from agilicus_api.model.mfa_enrollment_answer_result import MFAEnrollmentAnswerResult
from agilicus_api.model.mfa_enrollment_question import MFAEnrollmentQuestion
from agilicus_api.model.mfa_enrollment_question_input import MFAEnrollmentQuestionInput
from agilicus_api.model.mfa_enrollment_question_login_info import MFAEnrollmentQuestionLoginInfo
from agilicus_api.model.managed_upstream_identity_provider import ManagedUpstreamIdentityProvider
from agilicus_api.model.many_org_token_introspect_response import ManyOrgTokenIntrospectResponse
from agilicus_api.model.map_attributes_answer import MapAttributesAnswer
from agilicus_api.model.map_attributes_answer_result import MapAttributesAnswerResult
from agilicus_api.model.map_attributes_question import MapAttributesQuestion
from agilicus_api.model.map_attributes_question_input import MapAttributesQuestionInput
from agilicus_api.model.map_attributes_question_login_info import MapAttributesQuestionLoginInfo
from agilicus_api.model.mapped_attributes import MappedAttributes
from agilicus_api.model.message import Message
from agilicus_api.model.message_action import MessageAction
from agilicus_api.model.message_endpoint import MessageEndpoint
from agilicus_api.model.message_endpoint_metadata import MessageEndpointMetadata
from agilicus_api.model.message_endpoint_spec import MessageEndpointSpec
from agilicus_api.model.message_endpoint_type import MessageEndpointType
from agilicus_api.model.message_endpoint_type_web_push import MessageEndpointTypeWebPush
from agilicus_api.model.message_endpoints_config import MessageEndpointsConfig
from agilicus_api.model.metadata_with_id import MetadataWithId
from agilicus_api.model.metadata_with_id_all_of import MetadataWithIdAllOf
from agilicus_api.model.metadata_with_only_id import MetadataWithOnlyId
from agilicus_api.model.next_page_email import NextPageEmail
from agilicus_api.model.oidc_auth_config import OIDCAuthConfig
from agilicus_api.model.oidc_auth_path_config import OIDCAuthPathConfig
from agilicus_api.model.oidc_auth_uri import OIDCAuthURI
from agilicus_api.model.oidc_content_type import OIDCContentType
from agilicus_api.model.oidc_proxy_config import OIDCProxyConfig
from agilicus_api.model.oidc_proxy_content_manipulation import OIDCProxyContentManipulation
from agilicus_api.model.oidc_proxy_domain_mapping import OIDCProxyDomainMapping
from agilicus_api.model.oidc_proxy_domain_name_mapping import OIDCProxyDomainNameMapping
from agilicus_api.model.oidc_proxy_domain_substitution import OIDCProxyDomainSubstitution
from agilicus_api.model.oidc_proxy_header import OIDCProxyHeader
from agilicus_api.model.oidc_proxy_header_mapping import OIDCProxyHeaderMapping
from agilicus_api.model.oidc_proxy_header_name import OIDCProxyHeaderName
from agilicus_api.model.oidc_proxy_header_override import OIDCProxyHeaderOverride
from agilicus_api.model.oidc_proxy_header_rewrite_filter import OIDCProxyHeaderRewriteFilter
from agilicus_api.model.oidc_proxy_header_user_config import OIDCProxyHeaderUserConfig
from agilicus_api.model.oidc_proxy_scope import OIDCProxyScope
from agilicus_api.model.oidc_proxy_standard_header import OIDCProxyStandardHeader
from agilicus_api.model.oidc_proxy_upstream_config import OIDCProxyUpstreamConfig
from agilicus_api.model.oidc_upstream_identity_provider import OIDCUpstreamIdentityProvider
from agilicus_api.model.org_scope_patch_document import OrgScopePatchDocument
from agilicus_api.model.organisation import Organisation
from agilicus_api.model.organisation_admin import OrganisationAdmin
from agilicus_api.model.organisation_state_selector import OrganisationStateSelector
from agilicus_api.model.organisation_state_status import OrganisationStateStatus
from agilicus_api.model.organisation_status import OrganisationStatus
from agilicus_api.model.permitted_cross_domain_policies_settings import PermittedCrossDomainPoliciesSettings
from agilicus_api.model.policy import Policy
from agilicus_api.model.policy_condition import PolicyCondition
from agilicus_api.model.policy_group import PolicyGroup
from agilicus_api.model.policy_group_spec import PolicyGroupSpec
from agilicus_api.model.policy_rule import PolicyRule
from agilicus_api.model.policy_rule_spec import PolicyRuleSpec
from agilicus_api.model.policy_spec import PolicySpec
from agilicus_api.model.previous_page_email import PreviousPageEmail
from agilicus_api.model.raw_token import RawToken
from agilicus_api.model.referrer_policy_settings import ReferrerPolicySettings
from agilicus_api.model.rendered_query_parameter import RenderedQueryParameter
from agilicus_api.model.rendered_resource_permissions import RenderedResourcePermissions
from agilicus_api.model.rendered_rule import RenderedRule
from agilicus_api.model.rendered_rule_body import RenderedRuleBody
from agilicus_api.model.replace_user_role_request import ReplaceUserRoleRequest
from agilicus_api.model.reset_mfa_challenge_method import ResetMFAChallengeMethod
from agilicus_api.model.reset_policy_request import ResetPolicyRequest
from agilicus_api.model.resource import Resource
from agilicus_api.model.resource_info import ResourceInfo
from agilicus_api.model.resource_permission import ResourcePermission
from agilicus_api.model.resource_permission_spec import ResourcePermissionSpec
from agilicus_api.model.resource_role import ResourceRole
from agilicus_api.model.resource_role_spec import ResourceRoleSpec
from agilicus_api.model.resource_session_stats import ResourceSessionStats
from agilicus_api.model.resource_spec import ResourceSpec
from agilicus_api.model.resource_stats import ResourceStats
from agilicus_api.model.resource_stats_metadata import ResourceStatsMetadata
from agilicus_api.model.resource_status import ResourceStatus
from agilicus_api.model.role import Role
from agilicus_api.model.role_list import RoleList
from agilicus_api.model.role_spec import RoleSpec
from agilicus_api.model.role_to_rule_entry import RoleToRuleEntry
from agilicus_api.model.role_to_rule_entry_spec import RoleToRuleEntrySpec
from agilicus_api.model.role_v2 import RoleV2
from agilicus_api.model.roles import Roles
from agilicus_api.model.rule import Rule
from agilicus_api.model.rule_query_body import RuleQueryBody
from agilicus_api.model.rule_query_body_json import RuleQueryBodyJSON
from agilicus_api.model.rule_query_parameter import RuleQueryParameter
from agilicus_api.model.rule_scope_enum import RuleScopeEnum
from agilicus_api.model.rule_spec import RuleSpec
from agilicus_api.model.rule_v2 import RuleV2
from agilicus_api.model.runtime_status import RuntimeStatus
from agilicus_api.model.secure_agent import SecureAgent
from agilicus_api.model.secure_agent_connector import SecureAgentConnector
from agilicus_api.model.secure_agent_connector_info import SecureAgentConnectorInfo
from agilicus_api.model.secure_agent_spec import SecureAgentSpec
from agilicus_api.model.secure_agent_status import SecureAgentStatus
from agilicus_api.model.service import Service
from agilicus_api.model.service_account import ServiceAccount
from agilicus_api.model.service_account_spec import ServiceAccountSpec
from agilicus_api.model.service_account_status import ServiceAccountStatus
from agilicus_api.model.service_forwarder import ServiceForwarder
from agilicus_api.model.service_forwarder_spec import ServiceForwarderSpec
from agilicus_api.model.service_forwarder_stats import ServiceForwarderStats
from agilicus_api.model.service_forwarder_stats_group import ServiceForwarderStatsGroup
from agilicus_api.model.service_forwarder_status import ServiceForwarderStatus
from agilicus_api.model.session import Session
from agilicus_api.model.sessions_spec import SessionsSpec
from agilicus_api.model.storage_region import StorageRegion
from agilicus_api.model.totp_enrollment import TOTPEnrollment
from agilicus_api.model.totp_enrollment_answer import TOTPEnrollmentAnswer
from agilicus_api.model.totp_enrollment_spec import TOTPEnrollmentSpec
from agilicus_api.model.totp_enrollment_status import TOTPEnrollmentStatus
from agilicus_api.model.time_interval_metrics import TimeIntervalMetrics
from agilicus_api.model.time_validity import TimeValidity
from agilicus_api.model.token import Token
from agilicus_api.model.token_introspect import TokenIntrospect
from agilicus_api.model.token_introspect_options import TokenIntrospectOptions
from agilicus_api.model.token_reissue_request import TokenReissueRequest
from agilicus_api.model.token_revoke import TokenRevoke
from agilicus_api.model.token_scope import TokenScope
from agilicus_api.model.token_validity import TokenValidity
from agilicus_api.model.uri_parameter_rewrite_filter import URIParameterRewriteFilter
from agilicus_api.model.upstream_group_excluded_entry import UpstreamGroupExcludedEntry
from agilicus_api.model.upstream_group_mapping import UpstreamGroupMapping
from agilicus_api.model.upstream_group_mapping_entry import UpstreamGroupMappingEntry
from agilicus_api.model.upstream_group_mapping_spec import UpstreamGroupMappingSpec
from agilicus_api.model.upstream_group_reconcile import UpstreamGroupReconcile
from agilicus_api.model.upstream_group_reconcile_response import UpstreamGroupReconcileResponse
from agilicus_api.model.upstream_user_identity import UpstreamUserIdentity
from agilicus_api.model.upstream_user_identity_spec import UpstreamUserIdentitySpec
from agilicus_api.model.usage_measurement import UsageMeasurement
from agilicus_api.model.usage_metric import UsageMetric
from agilicus_api.model.usage_metrics import UsageMetrics
from agilicus_api.model.user import User
from agilicus_api.model.user_application_access_info import UserApplicationAccessInfo
from agilicus_api.model.user_application_access_info_status import UserApplicationAccessInfoStatus
from agilicus_api.model.user_file_share_access_info import UserFileShareAccessInfo
from agilicus_api.model.user_file_share_access_info_status import UserFileShareAccessInfoStatus
from agilicus_api.model.user_identity import UserIdentity
from agilicus_api.model.user_identity_update import UserIdentityUpdate
from agilicus_api.model.user_identity_update_spec import UserIdentityUpdateSpec
from agilicus_api.model.user_info import UserInfo
from agilicus_api.model.user_member_of import UserMemberOf
from agilicus_api.model.user_metadata import UserMetadata
from agilicus_api.model.user_metadata_spec import UserMetadataSpec
from agilicus_api.model.user_metrics import UserMetrics
from agilicus_api.model.user_request_info import UserRequestInfo
from agilicus_api.model.user_request_info_spec import UserRequestInfoSpec
from agilicus_api.model.user_request_info_status import UserRequestInfoStatus
from agilicus_api.model.user_resource_access_info import UserResourceAccessInfo
from agilicus_api.model.user_resource_access_info_status import UserResourceAccessInfoStatus
from agilicus_api.model.user_roles import UserRoles
from agilicus_api.model.user_roles_for_an_org import UserRolesForAnOrg
from agilicus_api.model.user_session_identifiers import UserSessionIdentifiers
from agilicus_api.model.user_status_enum import UserStatusEnum
from agilicus_api.model.user_summary import UserSummary
from agilicus_api.model.web_auth_n_enrollment import WebAuthNEnrollment
from agilicus_api.model.web_auth_n_enrollment_answer import WebAuthNEnrollmentAnswer
from agilicus_api.model.web_auth_n_enrollment_spec import WebAuthNEnrollmentSpec
from agilicus_api.model.web_auth_n_enrollment_status import WebAuthNEnrollmentStatus
from agilicus_api.model.well_known_issuer_info import WellKnownIssuerInfo
from agilicus_api.model.whoami_request import WhoamiRequest
from agilicus_api.model.whoami_response import WhoamiResponse
from agilicus_api.model.workload_configuration import WorkloadConfiguration
from agilicus_api.model.x509_certificate import X509Certificate
from agilicus_api.model.x509_certificate_spec import X509CertificateSpec
from agilicus_api.model.x509_certificate_status import X509CertificateStatus
from agilicus_api.model.xss_settings import XSSSettings





import csv
import sys

import click

from typing import Any, Dict

from . import context


def update_if_not_none(obj: dict, new_values: dict):
    for k, v in new_values.items():
        if v is not None and (not isinstance(v, tuple) or len(v)):
            obj[k] = v


def strip_none(obj: dict):
    to_return = {}
    update_if_not_none(to_return, obj)
    return to_return


def update_if_present(object: dict, key, **kwargs):
    value = kwargs.get(key, None)
    if value is not None:
        object[key] = value


def pop_item_if_none(obj: dict, key=None):
    if not obj:
        return

    keys_to_pop = []
    if key is not None:
        if obj.get(key, True) is None:
            keys_to_pop.append(key)
    else:
        for k, v in obj.items():
            if v is None:
                keys_to_pop.append(k)
    for k in keys_to_pop:
        obj.pop(k)
    return obj


def build_updated_model(klass, model, new_values):
    model_dict = model.to_dict()
    update_if_not_none(model_dict, new_values)
    return klass(**model_dict)


def update_org_from_input_or_ctx(params, *args, **kwargs):
    org_id = get_org_from_input_or_ctx(*args, **kwargs)
    if org_id is not None:
        params["org_id"] = org_id
    else:
        params.pop("org_id", None)


def get_org_from_input_or_ctx(ctx, org_id=None, **kwargs):
    if org_id is None:
        token = context.get_token(ctx)
        org_id = context.get_org_id(ctx, token)

    # Treat an empty-string org id like None so that we can query all if necessary
    if org_id == "":
        org_id = None

    return org_id


def get_user_id_from_input_or_ctx(ctx, user_id=None, **kwargs):
    if user_id is None:
        token = context.get_token(ctx)
        user_id = context.get_user_id(ctx, token)

    # Treat an empty-string like None so that we can query all if necessary
    if user_id == "":
        user_id = None

    return user_id


def parse_csv_input(input_filename, parser):
    input_file = sys.stdin
    if input_filename != "-":
        input_file = open(input_filename, "r")

    results = list()
    with input_file:
        csv_input = csv.DictReader(input_file, delimiter=",", quotechar='"')

        for result_dict in csv_input:
            result = parser(result_dict)
            if result:
                results.append(result)

    return results


class SubObjectType(click.ParamType):
    """
    SubObjectType allows us to map an input value to a specific location in an object
    we're building up.
      `location` identifies the sub object to map to (e.g. metadata).
      `base_type` identifies the type we're wrapping (e.g. click.INT)
    use `get_object_by_location` to extract the k,v pairs for a specific location.
    """

    def __init__(self, location, base_type: click.ParamType):
        self.location = location
        self.base_type = base_type
        self.name = base_type.name
        self._value = None

    def convert(self, value, param, ctx):
        self._value = self.base_type(value, param, ctx)
        return self

    def value(self):
        return self._value


class SubObjectString(SubObjectType):
    def __init__(self, location):
        super().__init__(location, click.STRING)


class SubObjectInt(SubObjectType):
    def __init__(self, location):
        super().__init__(location, click.INT)


def get_objects_by_location(location, objects: Dict[str, Any]) -> Dict[str, Any]:
    result = {}
    for key, obj in objects.items():
        if not isinstance(obj, SubObjectType):
            continue
        if obj.location != location:
            continue

        val = obj.value()
        if val is None:
            continue

        result[key] = val

    return result



"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2021.06.02
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.permissions_api import PermissionsApi  # noqa: E501


class TestPermissionsApi(unittest.TestCase):
    """PermissionsApi unit test stubs"""

    def setUp(self):
        self.api = PermissionsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_bulk_delete_resource_permission(self):
        """Test case for bulk_delete_resource_permission

        Delete a set of resource permissions matching the resource id  # noqa: E501
        """
        pass

    def test_create_resource_permission(self):
        """Test case for create_resource_permission

        Create a ResourcePermission  # noqa: E501
        """
        pass

    def test_create_resource_role(self):
        """Test case for create_resource_role

        Create a ResourceRole  # noqa: E501
        """
        pass

    def test_delete_resource_permission(self):
        """Test case for delete_resource_permission

        Delete a ResourcePermission  # noqa: E501
        """
        pass

    def test_delete_resource_role(self):
        """Test case for delete_resource_role

        Delete a ResourceRole  # noqa: E501
        """
        pass

    def test_get_elevated_user_roles(self):
        """Test case for get_elevated_user_roles

        Get elevated roles for a user  # noqa: E501
        """
        pass

    def test_get_resource_permission(self):
        """Test case for get_resource_permission

        Get a ResourcePermission  # noqa: E501
        """
        pass

    def test_get_resource_role(self):
        """Test case for get_resource_role

        Get a ResourceRole  # noqa: E501
        """
        pass

    def test_list_elevated_user_roles(self):
        """Test case for list_elevated_user_roles

        List all elevated users and their roles  # noqa: E501
        """
        pass

    def test_list_resource_permissions(self):
        """Test case for list_resource_permissions

        List all ResourcePermissions  # noqa: E501
        """
        pass

    def test_list_resource_roles(self):
        """Test case for list_resource_roles

        List all ResourceRoles  # noqa: E501
        """
        pass

    def test_list_resource_roles_for_type(self):
        """Test case for list_resource_roles_for_type

        List all ResourceRoles  # noqa: E501
        """
        pass

    def test_replace_elevated_user_role(self):
        """Test case for replace_elevated_user_role

        Create or update an elevated user role  # noqa: E501
        """
        pass

    def test_replace_resource_permission(self):
        """Test case for replace_resource_permission

        Update a ResourcePermission  # noqa: E501
        """
        pass

    def test_replace_resource_role(self):
        """Test case for replace_resource_role

        Update a ResourceRole  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from unittest.mock import MagicMock

class AuditsApiMock:

    def __init__(self):
        self.mock_list_audits = MagicMock()
        self.mock_list_auth_records = MagicMock()

    def list_audits(self, *args, **kwargs):
        """
        This method mocks the original api AuditsApi.list_audits with MagicMock.
        """
        return self.mock_list_audits(self, *args, **kwargs)

    def list_auth_records(self, *args, **kwargs):
        """
        This method mocks the original api AuditsApi.list_auth_records with MagicMock.
        """
        return self.mock_list_auth_records(self, *args, **kwargs)

